from examples.first_model import user, Product
from model_behavior.assignments.task import Booking
from nested_models.examples.nested_model import address, delivery, comment
from serialization.examples.serialization import meet

if __name__ == "__main__":
    print(user)

    product_data: Product = Product(id=55, name="perfume", in_stock=True, price=499)
    print(product_data.model_dump_json())

    booking: Booking = Booking(user_id=10, room_id=20, nights=3, rate_per_night=10)
    print(booking.model_dump_json())

    print(address.model_dump_json())
    print(delivery.model_dump_json())
    print(comment.model_dump_json())

    print(meet.model_dump_json())
    print(meet.model_dump())



from typing import List

from pydantic import BaseModel


class Course(BaseModel):
    id: int
    name: str


class Module(BaseModel):
    id: int
    name: str
    description: str
    course: Course


class Lesson(BaseModel):
    id: int
    name: str
    description: str
    modules: List[Module]



from pydantic import BaseModel
from typing import Optional, List, Dict
from datetime import datetime


class User(BaseModel):
    id: int
    name: str
    is_active: bool


class Product(BaseModel):
    id: int
    name: str
    price: float
    in_stock: bool


class Cart(BaseModel):
    user_id: int
    items: List[Product]
    quantities: Dict[str, int]
    updated_at: Optional[datetime] = None
    created_at: Optional[datetime] = None


input_data = {"id": 10, "name": "meet kumar", "is_active": True}

user = User(**input_data)



from typing import Dict

from pydantic import BaseModel, field_validator, model_validator, computed_field


class User(BaseModel):
    name: str

    @field_validator("name")
    def validate_name(self, value) -> str:
        if len(value) < 3:
            raise ValueError("name must be at least 3 characters long")
        return value


class SignupData(BaseModel):
    email: str
    password: str
    confirm_password: str

    @model_validator(mode="after")
    def password_match(self, values) -> Dict[str, str]:
        if values["password"] != values["confirm_password"]:
            raise ValueError("password and confirmation password do not match")
        return values


class LoginAttempt(BaseModel):
    attempts: int
    max_attempts: int = 5

    @computed_field
    @property
    def attempt_number(self) -> bool:
        if self.attempts == self.max_attempts:
            raise ValueError("attempt number exceeds max attempt number")
        return True



from typing import Optional

from pydantic import BaseModel, Field


class Employee(BaseModel):
    id: int
    name: str = Field(
        ...,
        min_length=3,
        max_length=50,
        description="Employee Name",
        examples=["meet kumar"],
    )
    department: Optional[str] = "General"
    salary: float = Field(
        ...,
        ge=1_00_000.00,
        decimal_places=2,
        description="Employee Salary",
        examples=[50_000.00, 80_000.00, 1_00_000.00],
    )



from pydantic import BaseModel, Field, computed_field


class Booking(BaseModel):
    user_id: int = Field(...)
    room_id: int = Field(...)
    nights: int = Field(...)
    rate_per_night: float = Field(...)

    @computed_field
    @property
    def total_amount_per_night(self) -> float:
        return self.nights * self.rate_per_night



from typing import Dict, List

from pydantic import BaseModel


class Property(BaseModel):
    id: int
    name: str
    dimension: Dict[str, float] = None
    total_square_feet: float = None
    rate_per_square_feet: float = None
    is_sold: bool = False


class Owner(BaseModel):
    id: int
    name: str
    properties: List[Property]


bhopal = Property(
    id=1,
    name="bhopal",
    dimension={"length": 100, "width": 100, "height": 100},
    total_square_feet=2.0,
    rate_per_square_feet=1.0,
)

indore = Property(
    id=2,
    name="indore",
    dimension={"length": 100, "width": 100, "height": 100},
    total_square_feet=2.0,
    rate_per_square_feet=1.0,
)

meet = Owner(id=3, name="mee", properties=[bhopal, indore])



from typing import Optional, List

from pydantic import BaseModel


class Address(BaseModel):
    street: str
    city: str
    state: str


class Delivery(BaseModel):
    id: int
    address: Address


class Comment(BaseModel):
    id: int
    text: str
    replies: Optional[List["Comment"]] = None


##forward referencing
Comment.model_rebuild()
address = Address(street="shyamla hills", city="bhopal", state="madhya pradesh")

delivery = Delivery(id=1, address=address)

comment = Comment(
    id=1,
    text="hello world",
    replies=[Comment(id=2, text="hello world"), Comment(id=3, text="hello world")],
)
